# 3.10 VaR and Graphs Tests.R
# Illustrating graphs in R and introduce value-at-risk
# This illustration benefitted from the assistance of Ben Jackson
# rmarkdown::render("3.10 VaR and Graphs Tests.R", "word_document")
rm(list = ls())  # Take out the Environment "trash"
cat("\014") # Clear Console, making error checking easier.
while (!is.null(dev.list()))  dev.off() # Clear old plots
par(family = 'Times New Roman') # Globally set fonts for graphs
# Libraries
# ggplot2 - creating quality plots
# quantmod - pulling stock data
# kableExtra - aid in formatting some of our results
# knitr - create RMarkdown file producing a record of our results
# openxlsx - Read and write to excel files

Packages <- c("ggplot2", "quantmod", "kableExtra", "knitr", "openxlsx") 
if(length(setdiff(Packages, rownames(installed.packages()))) > 0) {
  install.packages(setdiff(Packages, rownames(installed.packages())))
} # Make sure libraries are installed on this computer
lapply(Packages, library, character.only=TRUE) # Load and attach libraries
rm(Packages)
# For illustration purposes only, it is not important to fully understand 
#   value-at-risk (VaR) at this point.
# The goal with this program is merely illustrative.
ConfidenceLevel <- 0.95 # Define the confidence interval
NumberOfObservations <- 252
Security1 <- "AMZN" # Ticker
Import <- TRUE # If TRUE, then import from yahoo, if FALSE, read in file
# Import historical stock data via quantmod (Security1 abbreviated S1)
if(Import == TRUE){
  S1Data <- getSymbols(Security1, auto.assign = FALSE)
# Though the line below evaluates to TRUE, contains many other features
  is.numeric(S1Data)
# Note data is type xts, an time series class like zoo, 
#  data in ascending order
  head(S1Data, 5)
  tail(S1Data, 5)
# Convert xts to numeric, strips index and returns only observation
  S1GenericData <- coredata(S1Data)
  is.numeric(S1GenericData)
  head(S1GenericData, 5)
  tail(S1GenericData, 5)
# Must use S1GenericData so AdjClose is not overwritten
  S1AdjPrices <- data.frame("Date" = index(S1Data), 
    "AdjClose" = S1GenericData[,6])
# Note the column is generic (no AAPL reference)
  head(S1AdjPrices, 5)
  tail(S1AdjPrices, 5)
# MS Excel will store date as Julian (base date is 12/30/1899)
  write.xlsx(S1AdjPrices, "S1AdjPrices.xlsx")
} else {
  S1AdjPrices <- read.xlsx(xlsxFile = "S1AdjPrices.xlsx", sheet = 1, 
    skipEmptyRows = FALSE)
# MS Excel base year is 12/30/1899  
  S1AdjPrices$Date <- as.Date(S1AdjPrices$Date, origin = "1899-12-30")
}
# Trim data based on NumberOfObservations
S1AdjPrices1 <- tail(S1AdjPrices, NumberOfObservations)
# Current price is used in analytic value-at-risk below
CurrentPrice <- S1AdjPrices1$AdjClose[NumberOfObservations]
# Convert to zoo object for analysis
S1AdjPrices1z <- zoo(S1AdjPrices1, as.Date(S1AdjPrices1$Date))
S1AdjPrices1z$Date <- NULL # Delete Date from zoo object
# Calculate continuously compounded returns
S1R = diff(log(as.numeric(S1AdjPrices1z$'AdjClose'))) # CC rates of return
S1FD = diff(as.numeric(S1AdjPrices1z$'AdjClose')) # First difference
# Lose one observation
A = NA # Need to append one due to loss of one in calculating returns
# Need date for appending first observation below
TDate <- as.Date(index(S1AdjPrices1z))
# zoo object same size as S1AdjPrices1z
S1R <- zoo(append(S1R, A, after = 0), TDate) 
# zoo object same size as S1AdjPrices1z
S1FD <- zoo(append(S1FD, A, after = 0), TDate) 
# Add the return column
S1z <- cbind(S1AdjPrices1z, S1R, S1FD)
LengthS1z = length(S1z$S1R)
for(i in 1:LengthS1z){
  if(i==1){
    S1z$S1TR[1] = 1.0
    S1z$S1TD[1] = 0.0
  } else {
    S1z$S1TR[i] = as.numeric(S1z$S1TR[i-1]) * exp(as.numeric(S1z$S1R[i])) 
    S1z$S1TD[i] = as.numeric(S1z$S1TD[i-1]) + as.numeric(S1z$S1FD[i]) 
  }
}
# Extract security 1 returns and first differences from zoo
S1R <- as.numeric(S1z$S1R)
S1FD <- as.numeric(S1z$S1FD)
# Remove first observation (it is NA)
S1R <- S1R[-1]
S1FD <- S1FD[-1]
# Compute daily mean, convert to numeric and ignore NAs
S1RMean <- mean(S1R, na.rm = TRUE)
S1RSD <- sd(S1R, na.rm = TRUE)
S1FDMean <- mean(S1FD, na.rm = TRUE)
S1FDSD <- sd(S1FD, na.rm = TRUE)
# Lose one observation at the beginning
NumberOfReturns <- length(S1R) 
# Width of each bin for histogram
S1RBinWidth <- S1RSD/15 
S1FDBinWidth <- S1FDSD/15 
# For Historical VaR, we must sort all of our observations
S1R <- sort(S1R)
S1FD <- sort(S1FD)
# VaR as a percent
HistoricalVaRR <- as.numeric(quantile(S1R, ConfidenceLevel))
# VaR as a Dollar Value
HistoricalVaRD <- as.numeric(quantile(S1FD, ConfidenceLevel))
# Illustrate text conversions as functions
# Note: NumberOfDigits = 2 sets the default values if nothing is passed 
DecimalToPercentInText <- function(x, NumberOfDigits = 2) {
  stopifnot(is.numeric(x)) # Make sure it is numeric
  x = x * 100 # Convert decimal to percent
  y = round(x, NumberOfDigits) # Round, not truncate, at appropriate level
# Numeric to text, with number of digits on RHS  
  z = format(y, nsmall = NumberOfDigits) 
  return(paste0(z, "%"))
}
DollarToDollarInText <- function(x, NumberOfDigits = 2) {
  stopifnot(is.numeric(x)) 
  y = round(x, NumberOfDigits)
  z = format(y, nsmall = NumberOfDigits)
  return(paste0("$", z))
}
TextHistoricalVaRR <- DecimalToPercentInText(HistoricalVaRR, 4)
TextHistoricalVaRD <- DollarToDollarInText(HistoricalVaRD, 4)
# Print values
print(TextHistoricalVaRR)
print(TextHistoricalVaRD)
# Returns graphed
# Categorize outccomes based on VaR confidence level (Return)
Segment <- NULL
for (i in 1:NumberOfReturns){
  if(S1R[i] < S1R[(1-ConfidenceLevel) * NumberOfReturns]){
    Segment[i] <- "Downside"
  }
  else if (S1R[i] > S1R[ConfidenceLevel * NumberOfReturns]){
    Segment[i] <- "Upside"
  }
  else if (S1R[i] >= 0 & S1R[i] <= S1R[ConfidenceLevel * NumberOfReturns]){
    Segment[i] <- "Positive"
  }
  else if (S1R[i]<=0 & S1R[i]>=S1R[(1-ConfidenceLevel) * NumberOfReturns]){
    Segment[i] <- "Negative"
  }
}
# Create data frame for graphing returns (GP denotes graph parameters)
S1RGP <- data.frame(CCReturn = S1R, Segment)
# Illustrate with histogram.
ggplot(data = S1RGP) +
  geom_histogram(aes(S1RGP[,1], fill = Segment), binwidth = S1RBinWidth) +
  geom_vline(xintercept = S1R[ConfidenceLevel * NumberOfReturns], 
    linetype = "dashed", color = "gray38", size = 1) +
  geom_vline(xintercept = S1R[(1-ConfidenceLevel) * NumberOfReturns], 
    linetype = "dashed", color = "gray38", size = 1) +
  scale_fill_manual(values=
    c('firebrick3', 'firebrick4', 'forestgreen', 'green3')) +
  labs(x = "Period Return (CC)", y = "Frequency", 
    title = paste0("Returns for ", Security1, 
    " (CC, %, Historical VaR)")) +
  theme(legend.title = element_blank(), 
    plot.title = element_text(hjust = 0.5)) +
  scale_x_continuous(labels = scales::percent, limits=c(-4*S1RSD, 4*S1RSD)) +
  scale_y_continuous(labels = scales::comma) +
  stat_function(
    fun = function(x, mean, sd, n, bw){
      dnorm(x = x, mean = mean, sd = sd) * n * bw
    }, args = c(mean = S1RMean, sd = S1RSD, n = NumberOfReturns, 
      bw = S1RBinWidth), 
    color = "gray38", size = 1.2)
# Dollar profits graphed
# Categorize outccomes based on VaR confidence level (Return)
Segment <- NULL
for (i in 1:NumberOfReturns){
  if(S1FD[i] < S1FD[(1-ConfidenceLevel) * NumberOfReturns]){
    Segment[i] <- "Downside"
  }
  else if (S1FD[i] > S1FD[ConfidenceLevel * NumberOfReturns]){
    Segment[i] <- "Upside"
  }
  else if (S1FD[i]>=0 & S1FD[i]<=S1FD[ConfidenceLevel * NumberOfReturns]){
    Segment[i] <- "Positive"
  }
  else if (S1FD[i]<=0 & S1FD[i]>=S1FD[(1-ConfidenceLevel) * NumberOfReturns]){
    Segment[i] <- "Negative"
  }
}
# Create data frame for graphing returns (GP denotes graph parameters)
S1FDGP <- data.frame(CCReturn = S1FD, Segment)
# Illustrate with histogram.
ggplot(data = S1FDGP) +
  geom_histogram(aes(S1FDGP[,1], fill = Segment), binwidth = S1FDBinWidth) +
  geom_vline(xintercept = S1FD[ConfidenceLevel * NumberOfReturns], 
    linetype = "dashed", color = "gray38", size = 1) +
  geom_vline(xintercept = S1FD[(1-ConfidenceLevel) * NumberOfReturns], 
    linetype = "dashed", color = "gray38", size = 1) +
  scale_fill_manual(values=c('firebrick3', 'firebrick4', 'forestgreen', 
    'green3')) +
  labs(x = "Period Profits ($)", y = "Frequency", 
    title = paste0("Period Profit for ", Security1, 
    " (Profit, $, Historical VaR)")) +
  theme(legend.title = element_blank(), 
    plot.title = element_text(hjust = 0.5)) +
  scale_x_continuous(labels = scales::dollar, 
    limits = c(-4*S1FDSD, 4*S1FDSD)) +
# Need to manipulate y labels eventually  
  scale_y_continuous(labels = scales::comma) +
  stat_function(
    fun = function(x, mean, sd, n, bw){
      dnorm(x = x, mean = mean, sd = sd) * n * bw
    }, args = c(mean = S1FDMean, sd = S1FDSD, n = NumberOfReturns, 
      bw = S1FDBinWidth), 
    color = "gray38", size = 1.2)
#
# Work on analytic value-at-risk based on normal distribution
#
Lambda = qnorm(ConfidenceLevel, 0.0, 1.0, lower.tail = TRUE, log.p = FALSE) 
# Note: Unit of measure is daily; hence, Horizon = 1 implies period is one 
#  trading day
Horizon = 1
ReturnVaRMean = S1RSD*(Horizon^0.5)*Lambda
DollarVaRMean = CurrentPrice * ReturnVaRMean
AnalyticVaRR = -S1RMean*Horizon + ReturnVaRMean
AnalyticVaRD = CurrentPrice * AnalyticVaRR
TextAnalyticVaRR <- DecimalToPercentInText(AnalyticVaRR, 4)
TextAnalyticVaRD <- DollarToDollarInText(AnalyticVaRD, 4)
# Print values
print(TextAnalyticVaRR)
print(TextAnalyticVaRD)
# Categorize outccomes based on Analytic VaR confidence level
Segment <- NULL
for (i in 1:NumberOfReturns){
  if(S1R[i] < -AnalyticVaRR){
    Segment[i] <- "Downside"
  }
  else if (S1R[i] > S1RMean*Horizon + ReturnVaRMean){
    Segment[i] <- "Upside"
  }
  else if (S1R[i] >= 0 & S1R[i] <= S1RMean*Horizon + ReturnVaRMean){
    Segment[i] <- "Positive"
  }
  else if (S1R[i] <= 0 & S1R[i] >= -AnalyticVaRR){
    Segment[i] <- "Negative"
  }
}
# Create data frame for graphing returns (GP denotes graph parameters)
S1RGP <- data.frame(CCReturn = S1R, Segment)
# Illustrate with histogram.
ggplot(data = S1RGP) +
  geom_histogram(aes(S1RGP[,1], fill = Segment), binwidth = S1RBinWidth) +
  geom_vline(xintercept = S1RMean*Horizon + AnalyticVaRR, 
    linetype = "dashed", color = "gray38", size = 1) +
  geom_vline(xintercept = -AnalyticVaRR, linetype = "dashed", 
    color = "gray38", size = 1) +
  scale_fill_manual(values=c('firebrick3', 'firebrick4', 'forestgreen', 
    'green3')) +
  labs(x = "Period Return (CC)", y = "Frequency", 
       title = paste0("Returns for ", Security1, " (CC, %, Analytic VaR)")) +
  theme(legend.title = element_blank(), 
    plot.title = element_text(hjust = 0.5)) +
  scale_x_continuous(labels = scales::percent, 
    limits = c(-4*S1RSD, 4*S1RSD)) +
  # Need to manipulate y labels eventually  
  scale_y_continuous(labels = scales::comma) +
  stat_function(
    fun = function(x, mean, sd, n, bw){
      dnorm(x = x, mean = mean, sd = sd) * n * bw
    }, args = c(mean = S1RMean, sd = S1RSD, n = NumberOfReturns, 
      bw = S1RBinWidth), 
      color = "gray38", size = 1.2)
# Categorize outccomes based on Analytic VaR confidence level
Segment <- NULL
for (i in 1:NumberOfReturns){
  if(S1FD[i] < -AnalyticVaRD){
    Segment[i] <- "Downside"
  }
  else if (S1FD[i] > S1FDMean*Horizon + AnalyticVaRD){
    Segment[i] <- "Upside"
  }
  else if (S1FD[i] >= 0 & S1FD[i] <= S1FDMean*Horizon + AnalyticVaRD){
    Segment[i] <- "Positive"
  }
  else if (S1FD[i] <= 0 & S1FD[i] >= -AnalyticVaRD){
    Segment[i] <- "Negative"
  }
}
# Create data frame for graphing returns (GP denotes graph parameters)
S1FDGP <- data.frame(DProfit = S1FD, Segment)
# Illustrate with histogram.
ggplot(data = S1FDGP) +
  geom_histogram(aes(S1FDGP[,1], fill = Segment), binwidth = S1FDBinWidth) +
  geom_vline(xintercept = S1FDMean*Horizon + AnalyticVaRD, 
    linetype = "dashed", color = "gray38", size = 1) +
  geom_vline(xintercept = -AnalyticVaRD, linetype = "dashed", 
    color = "gray38", size = 1) +
  scale_fill_manual(values=c('firebrick3', 'firebrick4', 'forestgreen', 
    'green3')) +
  labs(x = "Period Profit ($)", y = "Frequency", 
       title = paste0("Profit for ", Security1, 
         " (Profit, $, Analytic VaR)")) +
  theme(legend.title = element_blank(), 
    plot.title = element_text(hjust = 0.5)) +
  scale_x_continuous(labels = scales::dollar, 
    limits = c(-4*S1FDSD, 4*S1FDSD)) +
  # Need to manipulate y labels eventually  
  scale_y_continuous(labels = scales::comma) +
  stat_function(
    fun = function(x, mean, sd, n, bw){
      dnorm(x = x, mean = mean, sd = sd) * n * bw
    }, args = c(mean = S1FDMean, sd = S1FDSD, n = NumberOfReturns, 
      bw = S1FDBinWidth), 
    color = "gray38", size = 1.2)

